
/*
 * Last update 19-12-2008 
 * version: 1.0
 * 
 * This patch fixes some issues in the TelevisNet export scheduled job.
 *
 * 1 - enables logging for the job NET_DBCheckSizeVerifyData.
 *    Logs are saved in [StoredProcedureLog] table in the [_AcquiredDB_x] database.
 *    To view the log: SELECT ProcedureName, OperationDescription,[Timestamp]  FROM StoredProcedureLog order by Timestamp
 *
 * 2 -Modification to the  store procedure st_BuildView_Vw_HDataToSave that creates the view Vw_HDataToSave.
 *    the view is used in the export scheduled job: selecting the view was too slow when selection 
 *    from plant with more than ~7 historical network configuration.
 *
 * Script must be executed in the "_AcquiredDB_x" database.
 *   
 */


SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS ON 
GO

use _AcquiredDB_x

-- Log table creation
if exists (select * from dbo.sysobjects where id = object_id(N'[StoredProcedureLog]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
drop table [StoredProcedureLog]
GO

CREATE TABLE [StoredProcedureLog] (
	ID uniqueidentifier NOT NULL ,
	[Timestamp] datetime NOT NULL ,
	ProcedureName nvarchar (60) NOT NULL ,
	OperationDescription nvarchar (512) NOT NULL ,
	LogLevel int NOT NULL 
)
GO


-- Procedure for generic stored procedure logging
if exists (select * from dbo.sysobjects where id = object_id(N'[LogStoredProcedureAction]') and OBJECTPROPERTY(id, N'IsProcedure') = 1)
drop procedure [dbo].[LogStoredProcedureAction]
GO
Create Procedure [LogStoredProcedureAction]
    (
        @procedure_name nvarchar (60),
        @operation_description nvarchar (512),
        @log_level int
    )
As
    INSERT into StoredProcedureLog(ID, [Timestamp],ProcedureName, OperationDescription, LogLevel) 
    Values(NEWID(), GETDATE(), @procedure_name, @operation_description, @log_level);

RETURN (0)
GO



-- Alter original Televis Net procedure st_AcqDBCheckSizeSaveData, called in the job NET_DBCheckSizeVerifyData

ALTER        proc st_AcqDBCheckSizeSaveData
 @ACqDBName varchar(128),
 @CfgDBName varchar(128) 
AS


/*
* PROCEDURE st_AcqDBCheckSizeSaveData
* This StProc takes a series of actions which purpose is 
* to verify the size for ACQ Plant database and, if it is the case,
* to save requested old data to disk, Backup DBs and Truncate DB LOGs.
*
* Parameters:	@ACqDBName     Acquired NET DB Name
*              @CfgDBName     Config NET DB Name
*				
*
* RETURN CODES 0  No Error
*              10 BackupDir is null 
*              11 Unable to create BackupDir on disk
*	13 Failed to Execute bcp shell
*	14 Failed to rename or append save data file on disk
*
* NOTES: We retrieve DB Sizes constants from custom UDFs stored in MASTER Database
* The criteria to delete oldest data is to get the oldest week data stored, starting from
* oldest TIMESTAMP for ACQ DBs (same for Data and Alarms), from
* oldest TIMESTAMP for TWIN DB (just for Alarms, having no data), from
* oldest ValidateEndDate for CFG DBs (same for all Hist_* config tables)
* 
*/


/*
     Procedure was patched 18-12-2008. 
     The patch enables debug log, that are saved in [StoredProcedureLog] table

     procedure flow:
     
     - PROCEDURE STARTING UP SECTION
       set exportTargetDirectory
       set dataBaseSizeLimit = 800
     
     - DB BACKUP SECTION
       if DataBaseSize < (DataBaseSizeLimit) then
               ==>EXIT procedure
       else
               backup the database
        
     - DATE INTERVAL CALCULATION SECTION
       set interval = select a week starting from oldest date in Historicaldata and FlashAlarm
     
     if (interval falls accross two months) then 
          EXPORT SECTION (interval accross two months case)
          export data for the days in the first month
          set interval = select the days in the next month for the next section
       
     - EXPORT SECTION (normal case)
       export all the data in the interval
     
     - CONFIGURATION EXPORTING SECTION
     
     - BACKUP OF DATABASE LOG SECTION
     
     - SHINKING OF DATABASE  SECTION
     
     ==>EXIT procedure


*/



declare @OldestHDDate datetime
declare @OldestFADate datetime
declare @OldestDate datetime
declare @UpperDateLimit datetime
declare @UpperDateLimit1 datetime
declare @DBSizeLimit dec(10,2)
declare @BackupDir varchar(256)
declare @CurrentHDFile varchar(256)
declare @CurrentFAFile varchar(256)
declare @CurrentBackupFileName varchar(256)
declare @CurrentLogFileName varchar(256)
declare @Cmd varchar(4000)
declare @RetCode int
declare @DBSize dec(10,2)
declare @DummyInt int
declare @RecoveryModel varchar(20)
declare @FileRetentionMonths int
declare @DateFileToDelete datetime
declare @DummyDelete int
declare @ToDeleteHDFile varchar(256)
declare @ToDeleteFAFile varchar(256)
declare @PlantType int
declare @BCPOptionString varchar(256)

--logging
declare @logDetail int -- 0 = no logging; 
declare @storeNameForLogging varchar(60)
declare @errorDescriptionForLogging varchar(512)


--logging
set @PlantType = 0 -- Net
set @logDetail = 5
set @storeNameForLogging = 'st_AcqDBCheckSizeSaveData'
delete from StoredProcedureLog where ProcedureName = @storeNameForLogging -- empty previous logs of this stored procedure

-- bulk copy shell command options 
-- -T for trusted connection  or   -U usermname -P passwod 
-- originale: '-c -q -t"|"  ' 
set @BCPOptionString = '-c -q -t"|"  '

--set @ACqDBName='TWINAcqTest'
--set @CfgDBName='TWINCfgTest'
set @DBSizeLimit=master.dbo.fc_DefaultAcqDBSize()
set @FileRetentionMonths=master.dbo.fc_FileRetentionPeriod()
set @RecoveryModel=convert(varchar,DatabasePropertyEx(@AcqDBNAme,'Recovery'))


-- BEGIN PROCEDURE STARTING UP SECTION ##########
-- detect and set file backup destination directory 

--logging
if (@logDetail >= 0)
  exec LogStoredProcedureAction @storeNameForLogging, 'procedure started', 1

create table #DBSIZETABLE (dbsize dec)
set @cmd='use ' + @ACqDBName
set @cmd=@Cmd + ' insert #DBSIZETABLE exec sp_UsedSpace'
--print @cmd
exec (@Cmd)
select @Dbsize=dbsize from #DBSIZETABLE
drop table #DBSIZETABLE
--print 'DBSIZE:' + convert(varchar,@Dbsize)
set @Cmd='select setvalue into T_GENSETS from ' +@CfgDBName +'..Genericsettings 
          where [Name]=''backupdir'''
exec (@Cmd)


if (@logDetail >= 0)  --logging
  exec LogStoredProcedureAction @storeNameForLogging, 'T_GENSETS populated from Genericsettings', 1

select top 1 @BackupDir=Setvalue from T_GENSETS
drop table T_GENSETS


if @BackupDir is null 
  BEGIN
      if (@logDetail >= 0)  --logging
          exec LogStoredProcedureAction @storeNameForLogging, 'Err: stored exited because BackupDir is null', 1
          return(10)
  END
  else
  BEGIN
      set @Cmd='dir "' + @BackupDir + '"'
      exec @RetCode=master..xp_cmdshell @Cmd ,  no_output
      if @retCode<>0
            BEGIN
                set @Cmd='md "' + @BackupDir + '"'
                exec @RetCode=master..xp_cmdshell @Cmd ,  no_output
                if @RetCode<>0
                BEGIN
                      --logging
                      if (@logDetail >= 0)
                      exec LogStoredProcedureAction @storeNameForLogging, 'Err: stored exited because is unable to create BackupDir on disk', 1 
                      return(11)
                END
             END
   END
	
	
  --logging
  if (@logDetail >= 0)
  exec LogStoredProcedureAction @storeNameForLogging, 'Succesfully detected or created BackupDir on disk', 1 
  
-- END PROCEDURE STARTING UP SECTION ##########
  
  
				
  BEGIN -- MAIN PROCEDURE BLOCK
  
     -- DB BACKUP SECTION BEGIN #############
          
     create table #TEMPDIR (Nomefile varchar(256) null)
     set @Cmd='dir "' + @BackupDir + '\'+ @ACqDBName + '_?.bck" /b/od'
     --print @cmd
     insert #TEMPDIR exec master..xp_cmdshell @Cmd

     select @DummyInt=count(*) from #TEMPDIR where nomefile like @ACqDBName+'%'
     IF @DummyInt=0
           set @CurrentBackupFileName=@ACqDBName + '_1.bck'
     else if @DummyInt=1
           set @CurrentBackupFileName=@ACqDBName + '_2.bck'
     else
           select top 1 @CurrentBackupFileName=nomefile from #TEMPDIR
	
     drop table #TEMPDIR
     
     --logging
     if (@logDetail >= 0)
            exec LogStoredProcedureAction @storeNameForLogging, 'Succesfully selected backup file target.', 1 

     
     set @CurrentBackupFileName=@BackupDir + '\'+ @CurrentBackupFileName
     set @CurrentLogFileName= left(@CurrentBackupFileName,len(@CurrentBackupFileName)-3) + 'log'
     if @RecoveryModel <> 'SIMPLE'
     BEGIN
       BACKUP LOG @ACqDBName TO DISK = @CurrentLogFileName WITH   DESCRIPTION = 'Automatic Televis Log',INIT
       BACKUP LOG @ACqDBName  WITH TRUNCATE_ONLY 
     END

     -- If DB size is OK, shrink and exit the procedure
     if @DBSize<@DBSizeLimit 
     BEGIN
     
        if (@logDetail >= 0) --logging
              exec LogStoredProcedureAction @storeNameForLogging, 'DBsize is lower than limits but proceed with a SHRINK only ...', 1
               
        DBCC SHRINKDATABASE (@ACqDBName ,25,  NOTRUNCATE)
        -- Do backup anyway if a flag was set in generic settings
        IF EXISTS( Select * From _ConfigDB_x..genericsettings Where Name like 'EnableAlwaysDBBackup%' AND SetValue Like 'YES%')
        BACKUP DATABASE @ACqDBName TO DISK = @CurrentBackupFileName	 WITH   DESCRIPTION = 'Automatic Televis backup',INIT
        
        if (@logDetail >= 0) --logging
            exec LogStoredProcedureAction @storeNameForLogging, 'Procedure exited correctly bacause db size was lower than limit.', 1 
        return(0)
      END
      
      
      -- BACKUP
      if (@logDetail >= 0) -- logging
        exec LogStoredProcedureAction @storeNameForLogging, 'Procedure is performing ACqDB backup...', 1 

      BACKUP DATABASE @ACqDBName TO DISK = @CurrentBackupFileName WITH   DESCRIPTION = 'Automatic Televis backup',INIT
		
      if (@logDetail >= 0) -- logging
        exec LogStoredProcedureAction @storeNameForLogging, 'Procedure has correctly finished ACqDB backup', 1 

      -- DB BACKUP SECTION END ###################
      
      
      
      
      
      -- DATE INTERVAL CALCULATION SECTION BEGIN ###################
      -- the interval is a WEEK starting from the minimum date found.

      if exists (select * from dbo.sysobjects where id = object_id(N'[T_DATES]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
              drop table [T_DATES]
		
      set @Cmd='select min(timestamp)as MinDate into T_DATES from  ' +@AcqDBName +'..Historicaldata' 
      exec (@Cmd)
      select top 1 @OldestHDDate=MinDate from T_DATES
      drop table T_DATES

      set @Cmd='select min(timestamp)as MinDate into T_DATES from  ' +@AcqDBName +'..FlashAlarms' 
      exec (@Cmd)
      select top 1 @OldestFADate=MinDate from T_DATES
      drop table T_DATES

      --select @OldestHDDate= min(timestamp) from Historicaldata
      --select @OldestFADate = min(timestamp) from FlashAlarms
		
		
       if @OldestHDDate<@OldestFADate
             set @oldestDate=isnull(@OldestHDDate,@OldestFADate)
       ELSE
             SET @oldestDate	=isnull(@OldestFADate,@OldestHDDate)
       set @Upperdatelimit=dateadd(d,7,@oldestDate)
       
       -- DATE INTERVAL CALCULATION SECTION END ###################
       
       
       
       
       
       -- BEGIN EXPORT SECTION (interval accross two months case)  ###################
       -- interval date (a week) to export fall accross two months. proceed to export
       -- the days that fall in the first month. the next section will proceed for days in the next month
       if datediff(m,@oldestDate,@Upperdatelimit)>0 
       BEGIN
           SET @UpperDateLimit1=dateadd(d,-(day(@Upperdatelimit)-1),@Upperdatelimit)
           --HistoricalData	
        
            if (@logDetail >= 0) --logging
                  exec LogStoredProcedureAction @storeNameForLogging, 'Historical data block 1 saving started...', 1 
        	
            set @cmd=' exec ' + @ACqDBName +'.dbo.st_BuildView_Vw_HDataToSave '''+
            convert(varchar,@UpperDateLimit1,120) + ''','+ @CfgDBName 
				
            --logging
            set @errorDescriptionForLogging = 'Executing:' + convert(varchar(256),@Cmd); 
            if (@logDetail >= 0)
                    exec LogStoredProcedureAction @storeNameForLogging, @errorDescriptionForLogging, 1
                
            exec (@Cmd)
            set @Cmd='bcp "' + @ACqDBName + '..Vw_HDataToSave" out "'+
                      @BackupDir + '\TempData.txt"  ' + @BCPOptionString
            exec @RetCode=master..xp_cmdshell @Cmd , no_output
            if @RetCode<>0
            BEGIN 
                  --logging
                  set @errorDescriptionForLogging = 'Err: procedure is exiting bacause failed bcp shell ' + convert(varchar(256),@Cmd); 
                  if (@logDetail >= 0)
                        exec LogStoredProcedureAction @storeNameForLogging, @errorDescriptionForLogging, 1 
                  return(13)
            END
         

            set @UpperDateLimit1=dateadd(d,-1,@UpperDateLimit1) --just to generate correct file name
            set @DummyInt=month(@UpperDateLimit1)
            if @DummyInt <10
            BEGIN
                 set @CurrentHDFile='HistData_'+ convert(char(4),year(@UpperDateLimit1))+
                    '0' + convert(char(1),@DummyInt)+'.txt'
                 set @CurrentFAFile='FlashAlarms_'+ convert(char(4),year(@UpperDateLimit1))+
                    '0' + convert(char(1),@DummyInt)+'.txt'
            END
            else
            BEGIN
                 set @CurrentHDFile='HistData_'+ convert(char(4),year(@UpperDateLimit1))+
                 convert(char(2),@DummyInt)+'.txt'
                 set @CurrentFAFile='FlashAlarms_'+ convert(char(4),year(@UpperDateLimit1))+
	  convert(char(2),@DummyInt)+'.txt'
            END
            
            --logging
            set @errorDescriptionForLogging = 'File to export are ' + @CurrentHDFile +  ' and ' + @CurrentFAFile
            if (@logDetail >= 0) 
                  exec LogStoredProcedureAction @storeNameForLogging, @errorDescriptionForLogging , 1
                  
					
            -- Get file name to remove for FA and HD
            --
            set @DateFileToDelete = dateadd(m,-@FileRetentionMonths,@UpperDateLimit1)
            set @DummyDelete=month(@DateFileToDelete)
            if @DummyDelete <10 -- single or double digit months
            BEGIN
                  set @ToDeleteHDFile='HistData_'+ convert(char(4),year(@DateFileToDelete))+
                                      '0' + convert(char(1),@DummyDelete)+'.txt'
                  set @ToDeleteFAFile='FlashAlarms_'+ convert(char(4),year(@DateFileToDelete))+
                                      '0' + convert(char(1),@DummyDelete)+'.txt'
            END
            else
            BEGIN
                  set @ToDeleteHDFile='HistData_'+ convert(char(4),year(@DateFileToDelete))+
                  convert(char(2),@DummyDelete)+'.txt'
                  set @ToDeleteFAFile='FlashAlarms_'+ convert(char(4),year(@DateFileToDelete))+
                  convert(char(2),@DummyDelete)+'.txt'
            END
            
            set @errorDescriptionForLogging = 'File to delete are ' + @ToDeleteHDFile + ' and ' + @ToDeleteFAFile
            if (@logDetail >= 0) --logging
                  exec LogStoredProcedureAction @storeNameForLogging, @errorDescriptionForLogging , 1
            -- End file to Delete
				
            set @UpperDateLimit1=dateadd(d,1,@UpperDateLimit1)  --Correct date
            set @cmd='dir "' + @BackupDir + '\' + @CurrentHDFile  + '"'
            exec @RetCode=master..xp_cmdshell @Cmd ,  no_output
            if @RetCode=0
                    set @Cmd='copy "' + @BackupDir + '\' + @CurrentHDFile + '" + "' + @BackupDir + '\TempData.txt" "' + @BackupDir + '\' + @CurrentHDFile + '" /B'
            ELSe				
                    set @Cmd='move "' + @BackupDir + '\TempData.txt" "' + @BackupDir + '\' + @CurrentHDFile + '"'				
            exec @RetCode=master..xp_cmdshell @Cmd 	, no_output		
            if @RetCode<>0
            BEGIN 
               if (@logDetail >= 0) --logging
                  exec LogStoredProcedureAction @storeNameForLogging, 'Err: procedure is exiting bacause has failed to rename or append save data file on disk (HDFile)', 1 
               return(14)
            END
        
            -- rimuovo file vecchio 
            set @Cmd = 'del "'+@BackupDir + '\' + @ToDeleteHDFile+ '" /F'
            exec @RetCode=master..xp_cmdshell @Cmd 	, no_output


            set @Cmd='delete from '+@ACqDBName +'..historicaldata where [timestamp]<''' + convert(varchar,@UpperDateLimit1,120) + ''''
            exec (@Cmd)
            --delete from Twinacqtest..historicaldata where [timestamp]<@UpperDateLimit1
            --FlashAlarms		
            set @cmd=' exec ' + @ACqDBName +'.dbo.st_BuildView_Vw_FlashAlmsToSave '''+
            convert(varchar,@UpperDateLimit1,120) + ''','+ @CfgDBName 
            exec (@Cmd)
            set @Cmd='bcp "' + @ACqDBName + '..Vw_FlashAlmsToSave" out "'+
            @BackupDir + '\TempData.txt"  ' + @BCPOptionString
            exec @RetCode=master..xp_cmdshell @Cmd  ,no_output
            if @RetCode<>0
            BEGIN 
              if (@logDetail >= 0) --logging
                 exec LogStoredProcedureAction @storeNameForLogging, 'Err: procedure is exiting bacause failed bcp shell (Vw_FlashAlmsToSave)', 1 
              return(13)
            END
        
            set @cmd='dir "' + @BackupDir + '\' + @CurrentFAFile + '"'
            exec @RetCode=master..xp_cmdshell @Cmd ,  no_output
            if @RetCode=0
	  set @Cmd='copy "'+@BackupDir + '\' + @CurrentFAFile + '" + "' +	@BackupDir + '\TempData.txt" "' + @BackupDir + '\' + @CurrentFAFile + '" /B'
            ELSe				
                 set @Cmd='move "'+	@BackupDir + '\TempData.txt" "' + @BackupDir + '\' + @CurrentFAFile + '"'				
            exec @RetCode=master..xp_cmdshell @Cmd	, no_output		
            if @RetCode<>0 
            BEGIN 
                if (@logDetail >= 0) --logging
                    exec LogStoredProcedureAction @storeNameForLogging, 'Err: procedure is exiting bacause has failed to rename or append save data file on disk (FAFile)', 1 
                return(14)
            END
        
            -- remove old file 
            set @Cmd = 'del "'+@BackupDir + '\' + @ToDeleteFAFile+ '" /F'
            exec @RetCode=master..xp_cmdshell @Cmd , no_output

            set @Cmd='delete N from ' + @ACqDBName + '..FlashAlarms FA inner join ' +
            @ACqDBName + '..notifications N on
                          FA.Id_FlashAlarm=N.Ref_IdFlashAlarm
                          where FA.[timestamp]<''' + convert(varchar,@UpperDateLimit1,120) + ''''
            exec (@Cmd)
            set @Cmd='delete from '+@ACqDBName +'..FlashAlarms where [timestamp]<''' + convert(varchar,@UpperDateLimit1,120) + ''''
            exec (@Cmd)

            if (@logDetail >= 0) --logging
            exec LogStoredProcedureAction @storeNameForLogging, 'Historical data block 1 saving succesfully finished', 1 
       END
       -- END EXPORT SECTION (interval accross two months case)  ###################
			
			
       -- BEGIN EXPORT SECTION (normal case)  ###################
       -- the interval to export fall inside a single month. 
       -- proceed with normal export of that month
       BEGIN
            if (@logDetail >= 0) --logging
                 exec LogStoredProcedureAction @storeNameForLogging, 'Historical data block 2 saving started...', 1 
    
            --HistoricalData		
            set @cmd=' exec ' + @ACqDBName +'.dbo.st_BuildView_Vw_HDataToSave '''+
            convert(varchar,@UpperDateLimit,120)+ ''','+ @CfgDBName 
				
			if (@logDetail >= 0) --logging
                 exec LogStoredProcedureAction @storeNameForLogging, @cmd, 1 
    
			
            exec (@Cmd)
            set @Cmd='bcp "' + @ACqDBName + '..Vw_HDataToSave" out "'+
            @BackupDir + '\TempData.txt"  ' + @BCPOptionString
			if (@logDetail >= 0) --logging
                 exec LogStoredProcedureAction @storeNameForLogging, @Cmd, 1 
            exec @RetCode=master..xp_cmdshell @Cmd, no_output
            if @RetCode<>0 -- Error
            BEGIN 
                --logging
                set @errorDescriptionForLogging = 'Err: procedure is exiting bacause failed bcp shell ' + convert(varchar(256),@Cmd);
                if (@logDetail >= 0)
                        exec LogStoredProcedureAction @storeNameForLogging, @errorDescriptionForLogging , 1 
                return(13)
            END

            set @DummyInt=month(@UpperDateLimit)
            if @DummyInt <10
                set @CurrentHDFile='HistData_'+ convert(char(4),year(@UpperDateLimit))+
                                   '0' + convert(char(1),@DummyInt)+'.txt'
            else
                set @CurrentHDFile='HistData_'+ convert(char(4),year(@UpperDateLimit))+
                                   convert(char(2),@DummyInt)+'.txt'
            
            set @errorDescriptionForLogging = 'File to export are ' + @CurrentHDFile + ' and ' + @CurrentHDFile
            if (@logDetail >= 0) --logging
                  exec LogStoredProcedureAction @storeNameForLogging, @errorDescriptionForLogging , 1
              
							 
            -- Get file name to remove for FA and HD (fileRetentionMonths is 12 months)
            --
            set @DateFileToDelete = dateadd(m,-@FileRetentionMonths,@UpperDateLimit)
            set @DummyDelete=month(@DateFileToDelete)
            if @DummyDelete <10
            BEGIN
                set @ToDeleteHDFile='HistData_'+ convert(char(4),year(@DateFileToDelete))+
                                    '0' + convert(char(1),@DummyDelete)+'.txt'
                set @ToDeleteFAFile='FlashAlarms_'+ convert(char(4),year(@DateFileToDelete))+
                                    '0' + convert(char(1),@DummyDelete)+'.txt'
            END
            else
            BEGIN
                set @ToDeleteHDFile='HistData_'+ convert(char(4),year(@DateFileToDelete))+
                                     convert(char(2),@DummyDelete)+'.txt'
                set @ToDeleteFAFile='FlashAlarms_'+ convert(char(4),year(@DateFileToDelete))+
                                     convert(char(2),@DummyDelete)+'.txt'
            END
            set @errorDescriptionForLogging = 'File to delete are ' + @ToDeleteHDFile + ' and ' + @ToDeleteFAFile
            if (@logDetail >= 0) --logging
                  exec LogStoredProcedureAction @storeNameForLogging, @errorDescriptionForLogging , 1
              
            -- End file to Delete

            set @cmd='dir "' + @BackupDir + '\' + @CurrentHDFile + '"' 
            exec @RetCode=master..xp_cmdshell @Cmd,  no_output
            if @RetCode=0
                    set @Cmd='copy "'+@BackupDir + '\' + @CurrentHDFile + '" + "' + @BackupDir + 
                             '\TempData.txt" "' + @BackupDir + '\' + @CurrentHDFile + '" /B'
            ELSe				
                    set @Cmd='move "'+	@BackupDir + '\TempData.txt" "' + @BackupDir + '\' + @CurrentHDFile + '"'
	     --print @Cmd				
            exec @RetCode=master..xp_cmdshell @Cmd, no_output	
            if @RetCode<>0
            BEGIN 
                if (@logDetail >= 0) --logging
                  exec LogStoredProcedureAction @storeNameForLogging, 'Err: procedure is exiting bacause has failed to rename or append save data file on disk (HDFile)', 1 
                return(14)
            END
				
            -- remove old file
            set @Cmd = 'del "'+@BackupDir + '\' + @ToDeleteHDFile+ '" /F'
            exec @RetCode=master..xp_cmdshell @Cmd , no_output
				
            set @Cmd='delete from '+@ACqDBName +'..historicaldata where [timestamp]<''' + convert(varchar,@UpperDateLimit,120) + ''''
            exec (@Cmd)
            --delete from Twinacqtest..historicaldata where [timestamp]<@UpperDateLimit

            --FlashAlarms		
            set @cmd=' exec ' + @ACqDBName +'.dbo.st_BuildView_Vw_FlashAlmsToSave '''+
            convert(varchar,@UpperDateLimit,120) + ''','+ @CfgDBName 
            --print @cmd
            exec (@Cmd)
            set @Cmd='bcp "' + @ACqDBName + '..Vw_FlashAlmsToSave" out "'+ @BackupDir + 
                     '\TempData.txt"  ' + @BCPOptionString
            exec @RetCode=master..xp_cmdshell @Cmd ,no_output
            if @RetCode<>0
            BEGIN 
               --logging
               set @errorDescriptionForLogging = 'Executing:' + convert(varchar(256),@Cmd);
               if (@logDetail >= 0)
                        exec LogStoredProcedureAction @storeNameForLogging, @errorDescriptionForLogging, 1 
               return(13)
            END

            if @DummyInt <10
                 set @CurrentFAFile='FlashAlarms_'+ convert(char(4),year(@UpperDateLimit))+
                                    '0' + convert(char(1),@DummyInt)+'.txt'
            else
                 set @CurrentFAFile='FlashAlarms_'+ convert(char(4),year(@UpperDateLimit))+
                                    convert(char(2),@DummyInt)+'.txt'
            set @cmd='dir "' + @BackupDir + '\' + @CurrentFAFile + '"'
            exec @RetCode=master..xp_cmdshell @Cmd ,  no_output
            if @RetCode=0
                 set @Cmd='copy "'+@BackupDir + '\' + @CurrentFAFile + '" + "' + @BackupDir + 
                          '\TempData.txt" "' + @BackupDir + '\' + @CurrentFAFile + '" /B'
            ELSe				
                 set @Cmd='move "'+ @BackupDir + '\TempData.txt" "' + @BackupDir + 
                          '\' + @CurrentFAFile + '"'
            --print @Cmd				
            exec @RetCode=master..xp_cmdshell @Cmd , no_output		
            if @RetCode<>0
            BEGIN 
                if (@logDetail >= 0) --logging
                  exec LogStoredProcedureAction @storeNameForLogging, 'Err: procedure is exiting bacause has failed to rename or append save data file on disk (FAFile)', 1 
                return(14)
            END
				
            -- remove old file
            set @Cmd = 'del "'+@BackupDir + '\' + @ToDeleteFAFile+ '" /F'
            exec @RetCode=master..xp_cmdshell @Cmd	, no_output

            set @Cmd='delete N from ' + @ACqDBName + '..FlashAlarms FA inner join ' +
                     @ACqDBName + '..notifications N on 
                     FA.Id_FlashAlarm=N.Ref_IdFlashAlarm
                     where FA.[timestamp]<''' + convert(varchar,@UpperDateLimit,120) + ''''
            exec (@Cmd)
            set @Cmd='delete from '+@ACqDBName +'..FlashAlarms where [timestamp]<''' + convert(varchar,@UpperDateLimit,120) + ''''
            exec (@Cmd)

            --logging
            if (@logDetail >= 0)
            exec LogStoredProcedureAction @storeNameForLogging, 'Historical data block 2 saving succesfully finished.', 1

        END
        -- END EXPORT SECTION (normal case)  ###################



        -- BEGIN CONFIGURATION EXPORTING SECTION								
        BEGIN
		  
             --logging
             if (@logDetail >= 0)
                  exec LogStoredProcedureAction @storeNameForLogging, 'Historical plant configuration data saving to file started', 1
		
             -- Popolo la tabella T_VirtConfxVirtInstxVirtRes sul DBConfig
             --
             set @Cmd = ' exec ' + @CfgDBName+'.dbo.st_BuildTable_T_VirtConfxVirtInstxVirtRes'
             exec (@Cmd)

             -- Esporto il file VirtConfxVirtInst a partire dalla  tabella T_VirtConfxVirtInstxVirtRes
             --
             set @cmd = 'bcp "SELECT DISTINCT IDConfig,IDVirtuaInst,VIName,VIAddress from ' + @CfgDBName + 
                        '..T_VirtConfxVirtInstxVirtRes" queryout "'+ @BackupDir + 
                        '\VirtConfxVirtInst.txt"  ' + @BCPOptionString

             exec @RetCode=master..xp_cmdshell @Cmd ,no_output
             if @RetCode<>0 
             BEGIN
                --logging
                if (@logDetail >= 0)
                    exec LogStoredProcedureAction @storeNameForLogging, 'Err: procedure is exiting bacause failed saving file VirtConfxVirtInst', 1 
                return (15)
             END
		 	 
             -- Esporto il file VirtInstxVirtRes a partire dalla  tabella T_VirtConfxVirtInstxVirtRes
             --
             set @cmd = 'bcp "SELECT DISTINCT IDVirtuaInst,IDVirtRes,VRName,VRMeasureUnit,VRResType from ' + @CfgDBName + '..T_VirtConfxVirtInstxVirtRes" queryout "'+
             @BackupDir + '\VirtInstxVirtRes.txt"  ' + @BCPOptionString

             exec @RetCode=master..xp_cmdshell @Cmd ,no_output
             if @RetCode<>0 
             BEGIN
                 if (@logDetail >= 0) --logging
                      exec LogStoredProcedureAction @storeNameForLogging, 'Err: procedure is exiting bacause failed saving file VirtInstxVirtRes', 1
	  return (16)
             END

             -- Esporto la tabella VirtualConfHist
             --
             set @cmd = 'bcp "SELECT IDNetConfig,TimeStamp,StartVirtInstrID,EndVirtInstrID from '+ @CfgDBName + 
                        '.dbo.VirtualConfHist order by TimeStamp asc " queryout "'+ @BackupDir + 
                        '\VirtConfHist.txt"  ' + @BCPOptionString
             exec @RetCode=master..xp_cmdshell @Cmd  ,no_output
             if @RetCode<>0 
             BEGIN 
	--logging
               if (@logDetail >= 0)
                   exec LogStoredProcedureAction @storeNameForLogging, 'Err: procedure is exiting bacause failed exporting to file the table VirtualConfHist', 1
               return(17)
             END
		 	
             -- Export nome, descrizione  e tipo  di impianto (NET = 1)
             --
             set @cmd = 'bcp "SELECT G1.SetValue PlantName ,G2.SetValue Notes, '+convert(varchar,@PlantType)+
                        ' from '+@CfgDBName+'.dbo.GenericSettings G1,'+@CfgDBName+
                        '.dbo.GenericSettings G2 where G1.Name Like ''PlantName%'' and G2.name Like ''Notes%'' " queryout "'+ @BackupDir + 
                        '\PlantName.txt"  ' + @BCPOptionString
             exec @RetCode=master..xp_cmdshell @Cmd  ,no_output
             if @RetCode<>0 
             BEGIN 
                if (@logDetail >= 0)
                     exec LogStoredProcedureAction @storeNameForLogging, 'Err: procedure is exiting bacause failed exporting name, description and type of the plant', 1
                return(18)
             END
		 	
             if (@logDetail >= 0) --logging
                  exec LogStoredProcedureAction @storeNameForLogging, 'Historical plant configuration data saving to file successfully finished.', 1
		
        END
        -- BEGIN CONFIGURATION EXPORTING SECTION #############
		
   END -- MAIN PROCEDURE BLOCK




-- BEGIN BACKUP OF DATABASE LOG SECTION  #############
if @RecoveryModel <> 'SIMPLE'
BEGIN
  if (@logDetail >= 0) --logging
      exec LogStoredProcedureAction @storeNameForLogging, ' RecoveryModel is not set to SIMPLE: starting to backup logs...', 1
  BACKUP LOG @ACqDBName TO DISK = @CurrentLogFileName	 WITH   DESCRIPTION = 'Automatic Televis Log',INIT
  BACKUP LOG @ACqDBName  WITH TRUNCATE_ONLY 		
  if (@logDetail >= 0) --logging
      exec LogStoredProcedureAction @storeNameForLogging, ' RecoveryModel is not set to SIMPLE: backup of logs succesfully completed.', 1
END
-- END BACKUP OF DATABASE LOG SECTION  #############



-- BEGIN SHINKING OF DATABASE  SECTION  #############
if (@logDetail >= 0) --logging
   exec LogStoredProcedureAction @storeNameForLogging, ' Final db shinking starting ... ', 1
DBCC SHRINKDATABASE (@ACqDBName ,25,  NOTRUNCATE)
if (@logDetail >= 0) --logging
      exec LogStoredProcedureAction @storeNameForLogging, ' Final db shinking succesfully completed.', 1
-- END SHINKING OF DATABASE  SECTION  #############    
      


-- EXIT CORRECTLY 
if (@logDetail >= 0) --logging
      exec LogStoredProcedureAction @storeNameForLogging, 'Procedure exited. without errors.', 1
RETURN (0)
GO



use _AcquiredDB_x
GO

 
ALTER             proc [st_BuildView_Vw_HDataToSave]
@UpperDate datetime,
@CfgDBName varchar(128)
as
/*
* PROCEDURE st_BuildView_Vw_HDataToSave
* This StProc builds the view to save on disk data from HistoricalData table
* by the criteria 'TimeStamp < @Upperdate
*
* Parameters:	@UpperDate	reference end date for rows to get
*				@CfgDBName	Config DB Name entwined to current Acquired Database
*
* RETURN CODES:	0	No Error
*				
*/

/*
 *   Patched 18-12-2008
 *   selecting from the created view was too slow when selecting
 *   from plant with many historical network configuration (~7 or more)
 *
 *   so  added 4 clauses reducing the number of combination in the join(instruments,resources):
 *             not VR.ValidateStartDate > ''' + convert(varchar,dateadd(d, 2, @upperdate),101)+ '''
 *         and not VI.ValidateStartDate > ''' + convert(varchar,dateadd(d, 2, @upperdate),101) + ''' 
 *         and not VR.ValidateEndDate < ''' + convert(varchar, dateadd(d, -10, @upperdate) ,101) + '''
 *         and not VI.ValidateEndDate < ''' + convert(varchar, dateadd(d, -10, @upperdate) ,101) + '''
 *
 *
 *   these clauses assume we are searching on instruments that are 
 *   valid in the interval [upperdate-10, upperdate+2], this is a safe interval that includes
 *   the export hard-coded interval that is a week [upperdate-7, upperdate]
 */

declare @CfgDBNameEXT varchar(128)
declare @MyCmd varchar(4000)
if exists (select * from dbo.sysobjects where id = object_id(N'[dbo].[Vw_HDataToSave]') and OBJECTPROPERTY(id, N'IsView') = 1)
drop view [dbo].[Vw_HDataToSave]

set @CfgDBNameEXT=master.dbo.fc_GetRegularIdentifier(@CfgDBName)
set @MyCmd='create view [dbo].[Vw_HDataToSave]
AS
SELECT  top 100 percent  convert(varchar,HD.[Timestamp],120) as RecTimeStamp,VI.ID_VirtualInstrument,HD.LogResID,
		convert(decimal(15,2),HD.[Value]) as RecValue 
FROM  dbo.HistoricalData HD inner join (
	  	' + @CfgDBNameEXT + '.dbo.VirtualResources VR INNER JOIN
        ' + @CfgDBNameEXT + '.dbo.VirtualInstruments VI ON VR.Ref_IDVirtualInstr = VI.ID_VirtualInstrument)
		ON HD.LogResID=VR.ID_VirtualResource and hd.[timestamp]>= vr.validatestartdate
where [timestamp]<''' + convert(varchar,@upperdate,101) + '''
order by rectimestamp
UNION
SELECT  top 100 percent  convert(varchar,HD.[Timestamp],120) as RecTimeStamp,VI.ID_VirtualInstrument,HD.LogResID,
		convert(decimal(15,2),HD.[Value]) as RecValue 
FROM  dbo.HistoricalData HD inner join (
	  	' + @CfgDBNameEXT + '.dbo.Hist_VirtualResources VR INNER JOIN
        ' + @CfgDBNameEXT + '.dbo.Hist_VirtualInstruments VI ON
              not VR.ValidateStartDate > ''' + convert(varchar,dateadd(d, 2, @upperdate),101) + '''
          and not VI.ValidateStartDate > ''' + convert(varchar,dateadd(d, 2, @upperdate),101) + ''' 
          and not VR.ValidateEndDate < ''' + convert(varchar, dateadd(d, -10, @upperdate) ,101) + '''
          and not VI.ValidateEndDate < ''' + convert(varchar, dateadd(d, -10, @upperdate) ,101) + '''
	  and VR.Ref_IDVirtualInstr = VI.ID_VirtualInstrument and
	      VR.ValidateStartDate>=VI.ValidateStartDate and
	      VR.ValidateEndDate<=VI.ValidateEndDate )
		ON HD.LogResID=VR.ID_VirtualResource and 
			hd.[timestamp]between vr.validatestartdate and vr.validateenddate
where [timestamp]<''' + convert(varchar,@upperdate,101) + '''
order by rectimestamp'

exec (@MyCmd)

RETURN (0)




GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS ON 
GO

